==> rmp-ipc-0.9.0.crate: rmp-ipc-0.9.0/src/ipc/stream_emitter.rs <==
use crate::error::Result;
use crate::error_event::{ErrorEventData, ERROR_EVENT_NAME};
use crate::events::event::Event;
use crate::events::payload::EventSendPayload;
use crate::ipc::context::Context;
use crate::protocol::AsyncProtocolStream;
use std::fmt::Debug;
use std::sync::Arc;
use tokio::io::AsyncWriteExt;
use tokio::sync::Mutex;

/// An abstraction over the raw tokio tcp stream
/// to emit events and share a connection across multiple
/// contexts.
pub struct StreamEmitter<S: AsyncProtocolStream> {
    stream: Arc<Mutex<S::OwnedSplitWriteHalf>>,
}

impl<S> Clone for StreamEmitter<S>
where
    S: AsyncProtocolStream,
{
    fn clone(&self) -> Self {
        Self {
            stream: Arc::clone(&self.stream),
        }
    }
}

impl<P> StreamEmitter<P>
where
    P: AsyncProtocolStream,
{
    pub fn new(stream: P::OwnedSplitWriteHalf) -> Self {
        Self {
            stream: Arc::new(Mutex::new(stream)),
        }
    }

    #[tracing::instrument(level = "trace", skip(self, data))]
    pub async fn _emit<T: EventSendPayload + Debug>(
        &self,
        namespace: Option<&str>,
        event: &str,
        data: T,
        res_id: Option<u64>,
    ) -> Result<EmitMetadata> {
        let data_bytes = data.to_payload_bytes()?;

        let event = if let Some(namespace) = namespace {
            Event::with_namespace(namespace.to_string(), event.to_string(), data_bytes, res_id)
        } else {
            Event::new(event.to_string(), data_bytes, res_id)
        };

        let event_id = event.id();

        let event_bytes = event.into_bytes()?;
        {
            let mut stream = self.stream.lock().await;
            (*stream).write_all(&event_bytes[..]).await?;
            tracing::trace!(bytes_len = event_bytes.len());
        }

        Ok(EmitMetadata::new(event_id))
    }

    /// Emits an event
    pub async fn emit<S: AsRef<str>, T: EventSendPayload + Debug>(
        &self,
        event: S,
        data: T,
    ) -> Result<EmitMetadata> {
        self._emit(None, event.as_ref(), data, None).await
    }

    /// Emits an event to a specific namespace
    pub async fn emit_to<S1: AsRef<str>, S2: AsRef<str>, T: EventSendPayload + Debug>(
        &self,
        namespace: S1,
        event: S2,
        data: T,
    ) -> Result<EmitMetadata> {
        self._emit(Some(namespace.as_ref()), event.as_ref(), data, None)
            .await
    }

    /// Emits a response to an event
    pub async fn emit_response<S: AsRef<str>, T: EventSendPayload + Debug>(
        &self,
        event_id: u64,
        event: S,
        data: T,
    ) -> Result<EmitMetadata> {
        self._emit(None, event.as_ref(), data, Some(event_id)).await
    }

    /// Emits a response to an event to a namespace
    pub async fn emit_response_to<S1: AsRef<str>, S2: AsRef<str>, T: EventSendPayload + Debug>(
        &self,
        event_id: u64,
        namespace: S1,
        event: S2,
        data: T,
    ) -> Result<EmitMetadata> {
        self._emit(
            Some(namespace.as_ref()),
            event.as_ref(),
            data,
            Some(event_id),
        )
        .await
    }
}

/// A metadata object returned after emitting an event.
/// This object can be used to wait for a response to an event.
pub struct EmitMetadata {
    message_id: u64,
}

impl EmitMetadata {
    pub(crate) fn new(message_id: u64) -> Self {
        Self { message_id }
    }

    /// The ID of the emitted message
    pub fn message_id(&self) -> u64 {
        self.message_id
    }

    /// Waits for a reply to the given message.
    #[tracing::instrument(skip(self, ctx), fields(self.message_id))]
    pub async fn await_reply<P: AsyncProtocolStream>(&self, ctx: &Context<P>) -> Result<Event> {
        let reply = ctx.await_reply(self.message_id).await?;
        if reply.name() == ERROR_EVENT_NAME {
            Err(reply.data::<ErrorEventData>()?.into())
        } else {
            Ok(reply)
        }
    }
}

==> rmp-ipc-0.11.1.crate: rmp-ipc-0.11.1/src/ipc/stream_emitter.rs <==
use crate::error::Result;
use crate::error_event::{ErrorEventData, ERROR_EVENT_NAME};
use crate::events::event::Event;
use crate::events::payload::EventSendPayload;
use crate::ipc::context::Context;
use crate::protocol::AsyncProtocolStream;
use std::sync::Arc;
use tokio::io::AsyncWriteExt;
use tokio::sync::Mutex;

/// An abstraction over the raw tokio tcp stream
/// to emit events and share a connection across multiple
/// contexts.
pub struct StreamEmitter<S: AsyncProtocolStream> {
    stream: Arc<Mutex<S::OwnedSplitWriteHalf>>,
}

impl<S> Clone for StreamEmitter<S>
where
    S: AsyncProtocolStream,
{
    fn clone(&self) -> Self {
        Self {
            stream: Arc::clone(&self.stream),
        }
    }
}

impl<P> StreamEmitter<P>
where
    P: AsyncProtocolStream,
{
    pub fn new(stream: P::OwnedSplitWriteHalf) -> Self {
        Self {
            stream: Arc::new(Mutex::new(stream)),
        }
    }

    #[tracing::instrument(level = "trace", skip(self, data))]
    pub async fn _emit<T: EventSendPayload>(
        &self,
        namespace: Option<&str>,
        event: &str,
        data: T,
        res_id: Option<u64>,
    ) -> Result<EmitMetadata> {
        let data_bytes = data.to_payload_bytes()?;

        let event = if let Some(namespace) = namespace {
            Event::with_namespace(namespace.to_string(), event.to_string(), data_bytes, res_id)
        } else {
            Event::new(event.to_string(), data_bytes, res_id)
        };

        let event_id = event.id();

        let event_bytes = event.into_bytes()?;
        {
            let mut stream = self.stream.lock().await;
            (*stream).write_all(&event_bytes[..]).await?;
            tracing::trace!(bytes_len = event_bytes.len());
        }

        Ok(EmitMetadata::new(event_id))
    }

    /// Emits an event
    pub async fn emit<S: AsRef<str>, T: EventSendPayload>(
        &self,
        event: S,
        data: T,
    ) -> Result<EmitMetadata> {
        self._emit(None, event.as_ref(), data, None).await
    }

    /// Emits an event to a specific namespace
    pub async fn emit_to<S1: AsRef<str>, S2: AsRef<str>, T: EventSendPayload>(
        &self,
        namespace: S1,
        event: S2,
        data: T,
    ) -> Result<EmitMetadata> {
        self._emit(Some(namespace.as_ref()), event.as_ref(), data, None)
            .await
    }

    /// Emits a response to an event
    pub async fn emit_response<S: AsRef<str>, T: EventSendPayload>(
        &self,
        event_id: u64,
        event: S,
        data: T,
    ) -> Result<EmitMetadata> {
        self._emit(None, event.as_ref(), data, Some(event_id)).await
    }

    /// Emits a response to an event to a namespace
    pub async fn emit_response_to<S1: AsRef<str>, S2: AsRef<str>, T: EventSendPayload>(
        &self,
        event_id: u64,
        namespace: S1,
        event: S2,
        data: T,
    ) -> Result<EmitMetadata> {
        self._emit(
            Some(namespace.as_ref()),
            event.as_ref(),
            data,
            Some(event_id),
        )
        .await
    }
}

/// A metadata object returned after emitting an event.
/// This object can be used to wait for a response to an event.
pub struct EmitMetadata {
    message_id: u64,
}

impl EmitMetadata {
    pub(crate) fn new(message_id: u64) -> Self {
        Self { message_id }
    }

    /// The ID of the emitted message
    pub fn message_id(&self) -> u64 {
        self.message_id
    }

    /// Waits for a reply to the given message.
    #[tracing::instrument(skip(self, ctx), fields(self.message_id))]
    pub async fn await_reply<P: AsyncProtocolStream>(&self, ctx: &Context<P>) -> Result<Event> {
        let reply = ctx.await_reply(self.message_id).await?;
        if reply.name() == ERROR_EVENT_NAME {
            Err(reply.data::<ErrorEventData>()?.into())
        } else {
            Ok(reply)
        }
    }
}
